#!ruby
#**********************************************************************
# def_rating_meibo.rb
# 競技かるたレーティングシステム(Ruby版)
# クラス定義 RatingMeibo           ; 成績名簿処理
# クラス定義 RatingGameResult      ; FILE_RESULTデータ処理
# クラス定義 RatingRanking         ; ランキング処理
#**********************************************************************

require 'nkf'
require 'fileutils'
require './def_rating_files'


# 成績名簿マスター（FILE_MEIBO）及び
# 対戦結果ファイル（FILE_PLAYERS）のデータ配列オフセット
MB_CODE = 0			# 1st. 選手コード
MB_NAME = 1			# 2nd. 選手名
MB_KAI  = 2			# 3rd. 所属会
MB_TOUR = 3			# 4th. 大会数
MB_GAME = 4			# 5th. 試合数
MB_WIN  = 5			# 6th. 勝ち数
MB_LOS  = 6			# 7th. 負け数
MB_NOP  = 7			# 8th. 不戦数
MB_PT   = 8			# 9th. 持ち点
MB_DAY  = 9			#10th. 最近出場日
MB_TDAY = 10		#11th. 大会開催日(FILE_PLAYERS)

MB_DNUM = 10		# FILE_MEIBO   １組のデータ数(1st～10th)
PL_DNUM = 11		# FILE_PLAYERS １組のデータ数(1st～11th)

# 選手コード付加後の
# 大会結果CSVファイル(FILE_RESULT)のデータ配列オフセット
# (RS_KAIより以降は対戦相手)
RS_CODE = 0			# 1st. 選手コード
RS_NAME = 1			# 2nd. 選手名
RS_KAI  = 2			# 3rd. 所属会

# 集計リスト及び
# 集計結果ファイル（FILE_SYUUKEI）のデータ配列オフセット
SY_CODE = 0			# 1st. 選手コード
SY_NAME = 1			# 2nd. 選手名
SY_TOUR = 2			# 3rd. 大会数
SY_WIN  = 3			# 4th. 勝ち数
SY_LOS  = 4			# 5th. 負け数
SY_NOP  = 5			# 6th. 不戦数
SY_PT   = 6			# 7th. 持ち点
SY_DAY  = 7			# 8th. 最近出場日

SY_DNUM = 7			# 集計リスト   １組のデータ数(1st～7th)
SK_DNUM = 8			# FILE_SYUUKEI １組のデータ数(1st～8th)

## ランクファイル（FILE_RANKFILE）のデータ配列オフセット
RK_RANK = 0			# 1st. ランク
RK_CODE = 1			# 2nd. 選手コード
RK_NAME = 2			# 3rd. 選手名
RK_KAI  = 3			# 4th. 所属会
RK_TOUR = 4			# 5th. 大会数
RK_GAME = 5			# 6th. 試合数
RK_WIN  = 6			# 7th. 勝ち数
RK_LOS  = 7			# 8th. 負け数
RK_NOP  = 8			# 9th. 不戦数
RK_PT   = 9			#10th. 持ち点
RK_DAY  = 10		#11th. 最近出場日

RK_DNUM = 11		# FILE_RANKFILE １組のデータ数(1st～11th)


#######################################################################
# クラスの定義

#======================================================================
# RatingMeibo       ; 成績名簿処理
#----------------------------------------------------------------------
# クラス変数
#  @@meibo    ：成績名簿マスター内容のリスト
#  @@namelist ：大会参加者の名前リスト
#  @@ddname   ：同姓同名リスト
#======================================================================
class RatingMeibo

# 初期化（initialize）を行う
	def initialize
		@@meibo     = []		# 成績名簿マスターのリスト(split済)
		@@meibo_dd  = []		# 同姓同名の選手の成績名簿のリスト
		@@meibo_ddk = []		# 同姓同名同会の選手の成績名簿のリスト
		@@dd_name   = []		# 同姓同名の選手名のリスト
		@@ddk_name  = []		# 同姓同名同会の選手名のリスト

		if FileTest.exist?(FILE_MEIBO) == true
			f = File.open(FILE_MEIBO, "r")
			s = f.gets
			while s = f.gets
				s.chomp!
				a = s.split(/\t/)
				@@meibo.push a
			end
			f.close
		else
			STDERR.puts FILE_MEIBO + "がありません。"
			exit 1
		end
		
	end

# 成績名簿マスター内容のリストを取得する
	def get_meibo
		return @@meibo
	end

# 同姓同名の選手の成績名簿を利用可能にする
	def new_meibo_dousei_doumei
		@@meibo_dd  = []
		@@meibo_ddk = []
		@@dd_name   = []
		@@ddk_name  = []

		# 同姓同名の選手名リストを作成する
		@@meibo.each { |a| @@dd_name.push a[MB_NAME]}
		@@dd_name = @@dd_name.group_by{|i| i}.reject{|k,v| v.one?}.keys
		
		# 同姓同名の選手を抽出した成績名簿を作成する
		@@meibo.each { |a| @@dd_name.include?(a[MB_NAME]) ? (@@meibo_dd.push a) : nil }

		# 同姓同名同会の選手名リストを作成する
		@@meibo.each { |a| @@ddk_name.push (a[MB_NAME] + "\t" + a[MB_KAI])}
		@@ddk_name = @@ddk_name.group_by{|i| i}.reject{|k,v| v.one?}.keys
		ddk = []
		@@ddk_name.each { |s|
			a = s.split(/\t/)
			ddk.push a[0]
		}
		@@ddk_name = ddk
		
		# 同姓同名同会の選手を抽出した成績名簿を作成する
		@@meibo.each { |a| @@ddk_name.include?(a[MB_NAME]) ? (@@meibo_ddk.push a) : nil }

		return @@meibo_dd
	end

# 同姓同名の選手の成績名簿内容のリストを取得する
	def get_meibo_dousei_doumei
		return @@meibo_dd
	end

# 同姓同名同会の選手の成績名簿内容のリストを取得する
	def get_meibo_dousei_doumei_doukai
		return @@meibo_ddk
	end

# 同姓同名の選手かチェックする
# 戻り値 true:同姓同名である、false:同姓同名でない
	def dousei_doumei_name?(pname = "")
		return @@dd_name.include?(pname) ? true : false
	end

# 同姓同名同会の選手かチェックする
# 戻り値 true:同姓同名同会である、false:同姓同名同会でない
	def dousei_doumei_doukai_name?(pname = "")
		return @@ddk_name.include?(pname) ? true : false
	end

# 名前１と名前２が同じ名前か棄権を考慮してチェックする。
# 戻り値 true:同じ名前、false:違う名前
	def eq_name?(pname1 = "", pname2 = "")
		pname1.sub!(/\(GIVEN\)/, "") if pname1 =~ /\(GIVEN\)/
		pname1.sub!(/\(OFFER\)/, "") if pname1 =~ /\(OFFER\)/
		pname2.sub!(/\(GIVEN\)/, "") if pname2 =~ /\(GIVEN\)/
		pname2.sub!(/\(OFFER\)/, "") if pname2 =~ /\(OFFER\)/
		return pname1 == pname2 ? true : false
	end

#[End of Class] RatingMeibo
end


#======================================================================
# RatingGameResult  ; FILE_RESULTデータ処理
#======================================================================
class RatingGameResult
	attr_accessor	:aite_kekka, :pname_kiken
	attr_accessor	:result

# initialize						; 初期化
	def initialize
		@aite_kekka = ""
		@pname_kiken = ""
		@result = []

		f = File.open(FILE_RESULT, "r")
		while s = f.gets
			s.chomp!
			@result.push s
		end
		f.close
	end


###################################################################
# ret = getcode(i, num, name, aite)
# FILE_RESULT 一覧表形式／○×形式とも使用
# i    : 対戦相手の配列番号(RESULTのレコード読み込み)
# num  : 選手コード
# name : 選手名
# aite : 対戦相手
# 戻り値
#   '0'   対戦相手が見つからなかった
#   code  対戦相手の選手コード
#-----------------------------------------------------------------
# (1) aite がコード化して記述のとき、そのコードを返す。
# (2) aite がコード化していないとき
#     対戦相手を調べてそのコードを返す。
#     対戦相手が見つからなければ '0' を返す。

	def getcode(i, num = "", tname = "", aite = "")
		if aite =~ /^[0-9]/
			if aite =~ /\(GIVEN\)/
				aite.sub!(/\(GIVEN\)/, "")
				@aite_kiken = "GIVEN"
			elsif aite =~ /\(OFFER\)/
				aite.sub!(/\(OFFER\)/, "")
				@aite_kiken = "OFFER"
			end
		end

		@result.each { |p|
			g = p.split(/,/)
			if g.length < i + 1
				next
			end

			#対戦相手(aite)及び対象の選手(tname/num)が共に一致したらcodeを返す
			pname = g[i]
			pname.sub!(/\(GIVEN\)/, "")
			pname.sub!(/\(OFFER\)/, "")

			if aite == g[MB_NAME] && (pname == tname || pname == num)
				return g[MB_CODE]
			end
		}
		return '0'
	end


###################################################################
# ret = getcode0(i, num, name, aite)
# FILE_RESULT ○×形式専用
# i    : 対戦相手の配列番号(RESULTのレコード読み込み)
# num  : 選手コード
# name : 選手名
# aite : 対戦相手
# 戻り値
#   '0'   対戦相手が見つからなかった
#   code  対戦相手の選手コード
#-----------------------------------------------------------------
# (1) aite がコード化して記述のとき、そのコードを返す。
# (2) aite がコード化していないとき
#     対戦相手を調べてそのコードを返す。
#     対戦相手が見つからなければ '0' を返す。

	def getcode0(i, num = "", tname = "", aite = "")
		@result.each { |p|
			g = p.split(/,/)

			#対戦相手(aite)及び対象の選手(tname/num)が共に一致したらcodeを返す
			pname = g[i]
			if pname =~ /\(GIVEN\)/
				pname.sub!(/\(GIVEN\)/, "")
				@pname_kiken = "GIVEN"
			elsif pname =~ /\(OFFER\)/
				pname.sub!(/\(OFFER\)/, "")
				@pname_kiken = "OFFER"
			else
				@pname_kiken = ""
			end

			if (aite == g[MB_NAME] || aite == g[MB_CODE]) && \
				(pname == tname || pname == num) then
				@aite_kekka = g[i+1]
				return g[MB_CODE]
			end
		}
		return '0'
	end

#[End of Class] RatingGameResult
end


#======================================================================
# RatingRanking     ; ランキング処理
#======================================================================
# クラス変数
#  @@m_un     ：ランキング非公開対象者の配列
#  @@m_un_max ：ランキング非公開対象者の数
#======================================================================
class RatingRanking
	@@m_un = []
	@@m_un_max = 0

# initialize						; 初期化
	def initialize
		if FileTest.exist?(FILE_UNRANK) == true
			unrank = File.open(FILE_UNRANK, "r")
			s = unrank.gets
			s.chomp!
			if s != "CODE-NO\t氏名\t所属会\t非公開設定日"
				File.open(FILE_ERROR, "a") { |f|
					f.puts "ランキング非公開対象者の読み込みに失敗しました。"
				}
				exit 1
			end

			@@m_un = []
			@@m_un_max = 0
			while s = unrank.gets
				s.chomp!
				a = s.split(/\t/)
				@@m_un.push a[MB_CODE]
				@@m_un_max += 1
			end
			unrank.close
		end
	end


#######################################################################
# 出場日が有効期限内かチェックする。
# ret = yuukou_kigen?(pday, tday)
# pday   : 前回の出場日 yyyy/mm/dd
# tday   : 今回の出場日 yyyy/mm/dd
# 戻り値 true:有効期限内(13か月後の同日まで)、false:期限切れ
#        ・2020年3月～2022年3月は新型コロナウイルス対応期間として
#          13か月の期限切れを停止して有効期限内とする。
#        ・2022年4月～2023年3月は激変緩和措置として13か月の
#          期限切れを37か月後の同日まで有効期限内とする。

	def yuukou_kigen?(pday = "", tday = "")
		m = pday.split(/\//)
		t = tday.split(/\//)

		# 2020年3月～2022年3月：13か月の期限切れを停止する
		ybase = 2020
		r1 = (2020 - ybase) * 12 + 3
		r2 = (2022 - ybase) * 12 + 3
		mt = (t[0].to_i - ybase) * 12 + t[1].to_i
		if r1 <= mt && mt <= r2
			r0 = r1 - 13
			mm = (m[0].to_i - 2020) * 12 + m[1].to_i
			return mm >= r0 ? true : false
		end

		# 2022年4月～2023年3月：37か月後の同日まで有効
		ybase = 2022
		r1 = (2022 - ybase) * 12 + 4
		r2 = (2023 - ybase) * 12 + 3
		mt = (t[0].to_i - ybase) * 12 + t[1].to_i
		if r1 <= mt && mt <= r2
			dm = (t[0].to_i - m[0].to_i) * 12 + (t[1].to_i - m[1].to_i)
			dd = t[2].to_i - m[2].to_i
			return dm <= 36 || (dm == 37 && dd <= 0) ? true : false
		end

		# 通常の判定処理：13か月後の同日まで有効
		dm = (t[0].to_i - m[0].to_i) * 12 + (t[1].to_i - m[1].to_i)
		dd = t[2].to_i - m[2].to_i
		return dm <= 12 || (dm == 13 && dd <= 0) ? true : false
	end


#######################################################################
# ランキング非公開対象者かチェックする
# ret = unranking_player?(pcode)
# pcode  : 選手コード
# @m_un  : ランキング非公開対象者の配列
# 戻り値 true:非公開、false:公開

	def unranking_player?(pcode = "")
		return @@m_un.include?(pcode) ? true : false
	end

#[End of Class] RatingRanking
end

