#!ruby
#**********************************************************************
# ruby p2_gamesort0.rb
#----------------------------------------------------------------------
# コード化された順序付き対戦記録(FILE_GAME)を作成する（○×形式用）
# 入力
#   FILE_RESULT     選手コードを付与した大会結果CSVファイル
# 出力
#   FILE_GAME       コード化された順序付き対戦記録
#   FILE_ERROR      エラー記録ファイル（エラー時）
#----------------------------------------------------------------------
# FILE_GAME の記録形式
# 勝者、敗者は選手コードで示す。
#   不戦："%d\t%s\t-"          ,回戦,勝者
#   棄権："%d\t%s\t%s\tAbsence",回戦,勝者,敗者
#   対戦："%d\t%s\t%s"         ,回戦,勝者,敗者
#----------------------------------------------------------------------

require './def_rating_files'
require './def_rating_meibo'


#######################################################################
# メインプログラム

err = 0
game = []
result_db = RatingGameResult.new

#----------------------------------------------------------------------
# FILE_RESULT のデータ整合性確認
#----------------------------------------------------------------------
result_db.result.each { |s|
	a = s.split(/,/)
	pname = a[MB_NAME]

	i = 3
	while i < a.length - 1
		aite = a[i]
		kekka = a[i+1]
		aite_kiken = ""

		if aite =~ /^-/
			if kekka == '○'
				i += 2
				next
			else
				# 対象選手の相手のコードが見つからない、
				# または相手と対象選手が同じコード（同姓同名）
				err = 1
				File.open(FILE_ERROR, "a") { |f|
					f.printf("不戦勝が○になっていません。（%d回戦 %s）\n", (i-1)/2, pname)
				}
				exit 1
			end
		elsif aite =~ /\(GIVEN\)/
			aite.sub!(/\(GIVEN\)/, "")
			aite_kiken = "GIVEN"
		elsif aite =~ /\(OFFER\)/
			aite.sub!(/\(OFFER\)/, "")
			aite_kiken = "OFFER"
		else
			aite_kiken = ""
		end

		code = result_db.getcode0(i,a[MB_CODE],a[MB_NAME],aite)
		if aite_kiken == "GIVEN"
			if kekka != "○"
				# GIVENと結果〇が対になっていない
				err = 1
				File.open(FILE_ERROR, "a") { |f|
					f.printf("棄権結果に整合性がありません。（%d回戦 %s vs %s）\n", (i-1)/2, pname, aite)
				}
				exit 1
			end
			if result_db.pname_kiken != "OFFER"
				# OFFER/GIVENが対になっていない
				err = 1
				File.open(FILE_ERROR, "a") { |f|
					f.printf("棄権結果に整合性がありません。（%d回戦 %s vs %s）\n", (i-1)/2, pname, aite)
				}
				exit 1
			end
		elsif aite_kiken == "OFFER"
			if kekka != "×"
				# OFFERと結果×が対になっていない
				err = 1
				File.open(FILE_ERROR, "a") { |f|
					f.printf("棄権結果に整合性がありません。（%d回戦 %s vs %s）\n", (i-1)/2, pname, aite)
				}
				exit 1
			end
			if result_db.pname_kiken != "GIVEN"
				# OFFER/GIVENが対になっていない
				err = 1
				File.open(FILE_ERROR, "a") { |f|
					f.printf("棄権結果に整合性がありません。（%d回戦 %s vs %s）\n", (i-1)/2, pname, aite)
				}
				exit 1
			end
		else
			if result_db.pname_kiken != ""
				# OFFER/GIVENが対になっていない
				err = 1
				File.open(FILE_ERROR, "a") { |f|
					f.printf("棄権結果に整合性がありません。（%d回戦 %s vs %s）\n", (i-1)/2, pname, aite)
				}
				exit 1
			end
		end
		
		if code == '0' || code == a[MB_CODE]
			# 対象選手の相手のコードが見つからない、
			# または相手と対象選手が同じコード（同姓同名）
			err = 1
			File.open(FILE_ERROR, "a") { |f|
				f.puts "#{s}"
				f.puts "  ERR> #{aite}"
			}
			exit 1
		else
			if kekka == '○' && result_db.aite_kekka == '×'
			elsif kekka == '×' && result_db.aite_kekka == '○'
			elsif kekka == '○(GIVEN)' && result_db.aite_kekka == '×(OFFER)'
			elsif kekka == '×(OFFER)' && result_db.aite_kekka == '○(GIVEN)'
			else
				# ○×結果が対になっていない
				err = 1
				File.open(FILE_ERROR, "a") { |f|
					f.printf("○×結果に不整合があります。（%d回戦 %s vs %s）\n", (i-1)/2, pname, aite)
				}
				exit 1
			end
		end
		i += 2
	end
}


#----------------------------------------------------------------------
# FILE_GAME を作成する
#----------------------------------------------------------------------
result_db.result.each { |s|
	a = s.split(/,/)

	i = 3
	while i < a.length - 1
		if a[i+1] == "○"
			aite = a[i]

			if aite =~ /^-/
				# 不戦勝の結果
				game.push sprintf("%d\t%s\t-",(i-1)/2,a[MB_CODE])
			elsif aite =~ /\(GIVEN\)/
				aite.sub!(/\(GIVEN\)/, "")
				code = result_db.getcode(i,a[MB_CODE],a[MB_NAME],aite)
				if code == '0' || code == a[MB_CODE]
					# 対象選手の相手のコードが見つからない、
					# または相手と対象選手が同じコード（同姓同名）
					err = 1
					File.open(FILE_ERROR, "a") { |f|
						f.puts "#{s}"
						f.puts "  ERR> #{aite}"
					}
				else
					# 棄権の結果
					game.push sprintf("%d\t%s\t%s\tAbsence",(i-1)/2,a[MB_CODE],code)
				end
			else
				code = result_db.getcode(i,a[MB_CODE],a[MB_NAME],aite)
				if code == '0' || code == a[MB_CODE]
					# 対象選手の相手のコードが見つからない、
					# または相手と対象選手が同じコード（同姓同名）
					err = 1
					File.open(FILE_ERROR, "a") { |f|
						f.puts "#{s}"
						f.puts "  ERR> #{aite}"
					}
				else
					# 通常の対戦結果
					game.push sprintf("%d\t%s\t%s",(i-1)/2,a[MB_CODE],code)
				end
			end
		end
		i += 2
	end
}

# gameを昇順に並べ替えて、FILE_GAMEに出力する
game.sort!
f = File.open(FILE_GAME, "w")
game.each { |s|
	f.puts s
}
f.close

#----------------------------------------------------------------------
# 終了ステータス
#   0: 正常
#   1: エラー (対戦結果が不適切)
#----------------------------------------------------------------------
if err == 0
	exit 0
else
	exit 1
end

