#!ruby
#**********************************************************************
# ruby p2_gamesort1.rb
#----------------------------------------------------------------------
# コード化された順序付き対戦記録(FILE_GAME)を作成する（一覧表形式用）
# 入力
#   FILE_RESULT     選手コードを付与した大会結果CSVファイル
# 出力
#   FILE_GAME       コード化された順序付き対戦記録
#   FILE_ERROR      エラー記録ファイル（エラー時）
#----------------------------------------------------------------------
# FILE_GAME の記録形式
# 勝者、敗者は選手コードで示す。
#   不戦："%d\t%s\t-"          ,回戦,勝者
#   棄権："%d\t%s\t%s\tAbsence",回戦,勝者,敗者
#   対戦："%d\t%s\t%s"         ,回戦,勝者,敗者
#----------------------------------------------------------------------

require './def_rating_files'
require './def_rating_meibo'


#######################################################################
# メインプログラム

err = 0
game = []
result_db = RatingGameResult.new

#----------------------------------------------------------------------
# FILE_RESULT の各行末のデータ整合性確認
#----------------------------------------------------------------------
count_win = 0
result_db.result.each { |s|
	a = s.split(/,/)
	lastnum = a.length - 1

	pname = a[MB_NAME]
	aite = a[lastnum]
	aite_kiken = ""

	# 行末の'WIN'または各種記号の適切性を確認する
	if aite == 'WIN'
		count_win += 1
		if count_win > 1
			err = 1
			File.open(FILE_ERROR, "a") { |f|
				f.puts "'WIN'が複数個あります。"
			}
			exit 1
		else
			next
		end
	elsif aite =~ /^-/
		err = 1
		File.open(FILE_ERROR, "a") { |f|
			f.puts "行末に'-'があります。"
		}
		exit 1
	elsif aite =~ /\(GIVEN\)/
		aite.sub!(/\(GIVEN\)/, "")
		aite_kiken = "GIVEN"
	elsif aite =~ /\(OFFER\)/
		aite.sub!(/\(OFFER\)/, "")
		aite_kiken = "OFFER"
	end

	# 行末の対戦組合せの整合性を確認する
	find_name = 0
	result_db.result.each { |t|
		b = t.split(/,/)
		if aite == b[MB_NAME]
			if b[lastnum] =~ /\(GIVEN\)/
				if aite_kiken == "OFFER"
					b[lastnum].sub!(/\(GIVEN\)/, "")
				else
					# OFFER/GIVENが対になっていない
					err = 1
					File.open(FILE_ERROR, "a") { |f|
						f.puts "棄権結果に整合性がありません。（#{pname} vs #{aite}）"
					}
					exit 1
				end
			elsif b[lastnum] =~ /\(OFFER\)/
				if aite_kiken == "GIVEN"
					b[lastnum].sub!(/\(OFFER\)/, "")
				else
					# OFFER/GIVENが対になっていない
					err = 1
					File.open(FILE_ERROR, "a") { |f|
						f.puts "棄権結果に整合性がありません。（#{pname} vs #{aite}）"
					}
					exit 1
				end
			else
				if aite_kiken != ""
					# OFFER/GIVENの一方がない
					err = 1
					File.open(FILE_ERROR, "a") { |f|
						f.puts "棄権結果に整合性がありません。（#{pname} vs #{aite}）"
					}
					exit 1
				end
			end
		
			if pname == b[lastnum]
				find_name = 1
				break
			else
				# 相手の対戦組合せの選手名が異なっている
				err = 1
				File.open(FILE_ERROR, "a") { |f|
					f.puts "対戦組合せに整合性がありません。（#{pname} vs #{aite}）"
				}
				exit 1
			end
		end
	}

	if find_name == 0
		# 相手の対戦組合せの選手名が見つからなかった
		err = 1
		File.open(FILE_ERROR, "a") { |f|
			f.puts "対戦組合せに整合性がありません。（#{pname} vs #{aite}）"
		}
		exit 1
	end
}


#----------------------------------------------------------------------
# FILE_GAME を作成する
#----------------------------------------------------------------------
result_db.result.each { |s|
	a = s.split(/,/)

	# 勝った相手の範囲で繰り返して、対戦結果データをgameに入れる
	for i in 3...(a.length - 1)
		aite = a[i]

		if aite =~ /^-/
			# 不戦勝の結果
			game.push sprintf("%d\t%s\t-",i-2,a[MB_CODE])
		elsif aite =~ /\(GIVEN\)/
			aite.sub!(/\(GIVEN\)/, "")
			code = result_db.getcode(i,a[MB_CODE],a[MB_NAME],aite)
			if code == '0' || code == a[MB_CODE]
				# 対象選手の相手のコードが見つからない、
				# または相手と対象選手が同じコード（同姓同名）
				err = 1
				File.open(FILE_ERROR, "a") { |f|
					f.puts "#{s}"
					f.puts "  ERR> #{aite}"
				}
			else
				# 棄権の結果
				game.push sprintf("%d\t%s\t%s\tAbsence",i-2,a[MB_CODE],code)
			end
		else
			code = result_db.getcode(i,a[MB_CODE],a[MB_NAME],aite)
			if code == '0' || code == a[MB_CODE]
				# 対象選手の相手のコードが見つからない、
				# または相手と対象選手が同じコード（同姓同名）
				err = 1
				File.open(FILE_ERROR, "a") { |f|
					f.puts "#{s}"
					f.puts "  ERR> #{aite}"
				}
			else
				# 通常の対戦結果
				game.push sprintf("%d\t%s\t%s",i-2,a[MB_CODE],code)
			end
		end
	end
}

# gameを昇順に並べ替えて、FILE_GAMEに出力する
game.sort!
f = File.open(FILE_GAME, "w")
game.each { |s|
	f.puts s
}
f.close


#----------------------------------------------------------------------
# 終了ステータス
#   0: 正常
#   1: エラー (対戦結果が不適切)
#----------------------------------------------------------------------
if err == 0
	exit 0
else
	exit 1
end

