#!ruby
#**********************************************************************
# ruby p4_merge.rb CSVFILE
#----------------------------------------------------------------------
# レーティング集計結果を成績名簿マスター候補にマージする
# 入力
#   CSVFILE       大会結果CSVファイル
#   FILE_SYUUKEI  レーティング集計結果
#   FILE_MEIBO    成績名簿マスター
# 出力
#   FILE_MERGE    集計結果がマージされた成績名簿マスター候補
#----------------------------------------------------------------------

require 'fileutils'
require './def_rating_files'
require './def_rating_meibo'


#######################################################################
# メインプログラム

#######################################################################
## 引数チェック
## p1: csvfile  ex) 20200126-007-大阪.csv

errmes = "rating_sub.rb から実行してください。"

argc = ARGV.count
if argc != 1
	STDERR.puts errmes
	STDERR.puts ""
	exit 1
end

csvfile = ARGV[0]
if csvfile !~ /^[0-9]+-[0-9]+-.+\.[cC][sS][vV]$/
	STDERR.puts errmes
	STDERR.puts ""
	exit 1
end


#---------------------------------------------------------------------
# 集計結果を成績名簿マスター候補(merge)へ反映する
# ・SYUUKEI、MASTERともソート済み（選手コード昇順）と前提している。
#---------------------------------------------------------------------
syuukei = File.open(FILE_SYUUKEI, "r")
meibo   = File.open(FILE_MEIBO, "r")
merge   = File.open(FILE_MERGE, "w")

# タイトル行をコピーする
m = meibo.gets
merge.print m

while s = syuukei.gets
	s.chomp!
	a = s.split(/\t/)

	while m = meibo.gets
		m.chomp!
		b = m.split(/\t/)

		if a[SY_CODE] == b[MB_CODE]
			p1  = b[MB_CODE]
			p2  = b[MB_NAME]
			p3  = b[MB_KAI]
			p4  = a[SY_TOUR].to_i
			p5  = a[SY_WIN].to_i + a[SY_LOS].to_i
			p6  = a[SY_WIN].to_i
			p7  = a[SY_LOS].to_i
			p8  = a[SY_NOP].to_i
			p9  = a[SY_PT].to_i
			p10 = a[SY_DAY]

			line = sprintf("%s\t%s\t%s\t%d\t%d\t%d\t%d\t%d\t%d\t%s",p1,p2,p3,p4,p5,p6,p7,p8,p9,p10)
			merge.puts line
			break
		else
			merge.puts m
		end
	end
end


#---------------------------------------------------------------------
# 成績名簿マスターの残りを出力する。
#---------------------------------------------------------------------
while m = meibo.gets
	merge.print m
end

syuukei.close
meibo.close
merge.close

