#!ruby
#**********************************************************************
# ruby p5_meibosorter.rb CSVFILE
#----------------------------------------------------------------------
# ランクファイル（FILE_RANKFILE）を作成する
# 入力
#   FILE_MEIBO      成績名簿マスター
# 出力
#   FILE_SYUUKEI    レーティング集計結果
#   taikainame.d1   ランキング対象のリストファイル
#   taikainame.d2   ランキング対象外のリストファイル
#   taikainame.d3   持ち点有効期限切れのリストファイル
#   FILE_RANKFILE   逆順の.d1ファイル一覧
#----------------------------------------------------------------------

require 'fileutils'
require './def_rating_files'
require './def_rating_meibo'


#######################################################################
# メインプログラム

#######################################################################
## 引数チェック
## p1: csvfile  ex) 20200126-007-大阪.csv

errmes = "rating_sub.rb から実行してください。"

argc = ARGV.count
if argc != 1
	STDERR.puts errmes
	STDERR.puts ""
	exit 1
end

#----------------------------------------------------------------------
# 大会結果CSVファイルの読み込み及びバックアップファイル名作成
#----------------------------------------------------------------------
csvfile = ARGV[0]
if csvfile !~ /^[0-9]+-[0-9]+-.+\.[cC][sS][vV]$/
	STDERR.puts errmes
	STDERR.puts ""
	exit 1
end


#-------------------------------------------------------
# ランクファイル名の設定
#  meibo1: rank/xxx.d1     ランキング対象
#  meibo2: rank/xxx.d2     規定大会数不足または未勝利、ランキング非公開
#  meibo3: rank/xxx.d3     期限切れ
#  date  : yyyy/m/d        大会開催日
#-------------------------------------------------------
wkdir = RatingFiles.new
meibo1 = ""
meibo2 = ""
meibo3 = ""
date = ""
meibo1, meibo2, meibo3, date = wkdir.all_rankfile_name(csvfile)

rank = RatingRanking.new

#-------------------------------------------------------
# 成績名簿マスターの読み込み及び選手のランク分類
#-------------------------------------------------------
m1 = []
m2 = []
m3 = []

f = File.open(FILE_MEIBO, "r")
while s = f.gets
	s.chomp!
	a = s.split(/\t/)
	key_p = a[MB_PT].to_i
	key_w = a[MB_WIN].to_i
	key_l = 9999 - a[MB_LOS].to_i

	d = a[MB_DAY].split(/\//)
	date_y = d[0].to_i
	date_m = d[1].to_i
	date_d = d[2].to_i

	#-----------------------------------------------------
	# ソートキーをMASTERに付加したものを３つに分類する。
	# m1 ランキング対象のリスト
	# m2 ランキング対象外のリスト
	# m3 持ち点有効期限切れのリスト
	#-----------------------------------------------------
	key = sprintf("%04d%04d%04d%4d%02d%02d",key_p,key_w,key_l,date_y,date_m,date_d)
	line = sprintf("%s\t%s",key,s)
	if rank.yuukou_kigen?(a[MB_DAY],date) == false
		m3.push line
	elsif a[MB_TOUR].to_i < 7 || a[MB_WIN].to_i == 0
		m2.push line
	elsif rank.unranking_player?(a[MB_CODE]) == true
		m2.push line
	else
		m1.push line
	end
end
f.close

m1.sort!.reverse!
m2.sort!.reverse!
m3.sort!.reverse!


#----------------------------------------------------------------------
# meibo1の作成
#----------------------------------------------------------------------
f = File.open(meibo1, "w")
f.close
f = File.open(meibo1, "a")
n = 1
m1.each { |record|
	data = record.split(/\t/)
	datamax = data.length
	data[0] = n
	f.printf("%d", data[0])
	i = 1
	while i < datamax
		f.printf("\t%s", data[i])
		i += 1
	end
	f.printf("\n")
	n += 1
}
f.close
m1.clear

#----------------------------------------------------------------------
# meibo2の作成
#----------------------------------------------------------------------

f = File.open(meibo2, "w")
f.close
f = File.open(meibo2, "a")
n = 1
m2.each { |record|
	data = record.split(/\t/)
	datamax = data.length
	data[0] = n
	f.printf("%d", data[0])
	i = 1
	while i < datamax
		f.printf("\t%s", data[i])
		i += 1
	end
	f.printf("\n")
	n += 1
}
f.close
m2.clear


#----------------------------------------------------------------------
# meibo3の作成
#----------------------------------------------------------------------

f = File.open(meibo3, "w")
f.close
f = File.open(meibo3, "a")
n = 1
m3.each { |record|
	data = record.split(/\t/)
	datamax = data.length
	data[0] = n
	f.printf("%d", data[0])
	i = 1
	while i < datamax
		f.printf("\t%s", data[i])
		i += 1
	end
	f.printf("\n")
	n += 1
}
f.close
m3.clear


#----------------------------------------------------------------------
# FILE_RANKFILEの作成
# (逆順の.d1ファイル一覧を作成する)
#----------------------------------------------------------------------

d1 = []
d1 = wkdir.rankfile_d1_reverse_list

f = File.open(FILE_RANKFILE, "w")
f.close
f = File.open(FILE_RANKFILE, "a")
d1.each { |line|
	f.puts line
}
f.close

exit 0

